@@ -25,22 +25,33 @@ import droidurl
 import xml.parsers.expat as expat
 import logging
 
-logging.basicConfig(format='%(levelname)s:%(filename)s:%(message)s', level=logging.INFO)
+logging.basicConfig(format='%(levelname)s:%(filename)s:%(message)s',
+                    level=logging.INFO)
 
 
 class droidsample:
     # Base class for an Android sample to analyze
-    
-    def __init__(self, filename, output='/tmp/analysis', verbose=False, clear=False, enable_procyon=False, 
-                 disable_description=False, silent=False, no_kit_exception=False):
-        # Setup analysis of a given sample. This does not perform the analysis in itself
+
+    def __init__(self,
+                 config,
+                 filename,
+                 output='/tmp/analysis',
+                 verbose=False,
+                 clear=False,
+                 disable_description=False,
+                 silent=False,
+                 no_kit_exception=False,
+                 import_exodus=False):
+        # Setup analysis of a given sample.
+        # This does not perform the analysis in itself
 
         assert filename is not None, "Filename is invalid"
 
+        self.config = config
         self.absolute_filename = filename
         self.clear = clear
-        self.enable_procyon = enable_procyon
-        self.disable_description = disable_description # we need those for recursive calls to process_file
+        # we need those for recursive calls to process_file
+        self.disable_description = disable_description
         self.silent = silent
         self.no_kit_exception = no_kit_exception
         self.ziprar = None  # zip file or rar file file handle
@@ -48,20 +59,25 @@ class droidsample:
         if verbose:
             logging.getLogger().setLevel(logging.DEBUG)
 
-        sanitized_basename = droidutil.sanitize_filename(os.path.basename(filename))
-        if not silent:
-            logging.info("Filename: "+filename)
-        self.properties = droidproperties.droidproperties(samplename=sanitized_basename,
-                                                          sha256=droidutil.sha256sum(filename),
-                                                          verbose=verbose)
+        sanitized_basename = droidutil.sanitize_filename(
+            os.path.basename(filename))
+
+        self.properties = droidproperties.droidproperties(
+            config=self.config,
+            samplename=sanitized_basename,
+            sha256=droidutil.sha256sum(filename),
+            verbose=verbose,
+            import_exodus=import_exodus)
         logging.debug("SHA256: %s" % (self.properties.sha256))
 
-        """Computing the SHA1 of a file is only useful to help out the analyst. 
-        The digest is written to the automatic analysis/description of the sample. That
-        description is located in the output analysis directory, so obviously, if the
-        end-user specifies he wants the output analysis directory erased, he obviously
-        does not want to know about the SHA1 digest, and thus it's useless to compute it."""
-        if not clear: 
+        """Computing the SHA1 of a file is only useful to help out the analyst.
+        The digest is written to the automatic analysis/description
+        of the sample. That description is located in the output analysis
+        directory, so obviously, if the end-user specifies he wants
+        the output analysis directory erased, he obviously
+        does not want to know about the SHA1 digest, and
+        thus it's useless to compute it."""
+        if not clear:
             sha1 = droidutil.sha1sum(filename)
             logging.debug("SHA1: %s" % (sha1))
 
@@ -70,14 +86,15 @@ class droidsample:
                 filename=sanitized_basename,
                 hash=self.properties.sha256))
 
-        logging.debug("Output analysis directory: " + self.outdir)
+        logging.debug("Output dir: " + self.outdir)
 
         if os.path.exists(self.outdir):
             try:
-                shutil.rmtree(self.outdir, ignore_errors=droidutil.on_rm_tree_error)
+                shutil.rmtree(self.outdir,
+                              ignore_errors=droidutil.on_rm_tree_error)
                 os.makedirs(self.outdir)
             except RuntimeError:
-                logging.debug("Failed to remove directory - rmtree / RuntimeError")
+                logging.debug("Failed to remove directory RuntimeError")
         else:
             os.makedirs(self.outdir)
 
@@ -93,72 +110,92 @@ class droidsample:
 
     def unzip(self):
         """
-        This method will unzip/unrar the sample, and recursively unzip/unrar inner zips/rars.
-        If we are not removing the analysis directory (clearoutput option), then we also
+        This method will unzip/unrar the sample,
+        and recursively unzip/unrar inner zips/rars.
+        If we are not removing the analysis directory
+        (clearoutput option), then we also
         unzip the sample in outdir/unzipped subdirectory.
-        If the sample is password protected, we try 'infected' as password.
-        
-        Returns the file type of the sample: droidutil.<FILE CONSTANT> (UNKNOWN, APK, DEX, ...)
+        If the sample is password protected,
+        we try 'infected' as password.
+        Returns the file type of the sample: droidutil
+        <FILE CONSTANT> (UNKNOWN, APK, DEX, ...)
         """
         logging.debug("------------- Unzipping %s" % (self.absolute_filename))
-            
-        self.properties.filetype = droidutil.get_filetype(self.absolute_filename)
+
+        self.properties.filetype = droidutil.get_filetype(
+            self.absolute_filename)
 
         if self.properties.filetype == droidutil.ARM or \
            self.properties.filetype == droidutil.UNKNOWN or \
            self.properties.filetype == droidutil.DEX:
-            logging.debug("This is a %s. Nothing to unzip for %s" % (droidutil.str_filetype(self.properties.filetype), 
-                                                                     self.absolute_filename))
+            logging.debug("This is a %s. Nothing to unzip for %s"
+                          % (droidutil.str_filetype(self.properties.filetype),
+                             self.absolute_filename))
             return self.properties.filetype
 
         if self.properties.filetype == droidutil.ZIP or \
                 self.properties.filetype == droidutil.RAR:
             if self.properties.filetype == droidutil.ZIP:
-                self.ziprar = droidziprar.droidziprar(self.absolute_filename,
-                                                      zipmode=True, verbose=self.verbose)
+                self.ziprar = droidziprar.droidziprar(
+                    self.absolute_filename,
+                    zipmode=True, verbose=self.verbose)
             else:
-                self.ziprar = droidziprar.droidziprar(self.absolute_filename,
-                                                      zipmode=False, verbose=self.verbose)
+                self.ziprar = droidziprar.droidziprar(
+                    self.absolute_filename,
+                    zipmode=False, verbose=self.verbose)
             if self.ziprar.handle is None:
                 self.properties.filetype = droidutil.UNKNOWN  # damaged zip/rar
-                logging.debug("We are unable to unzip/unrar %s because of errors" % (self.absolute_filename))
+                logging.debug("Unable to unzip/unrar %s because of errors"
+                              % self.absolute_filename)
                 return droidutil.UNKNOWN
             # Now, we know self.ziprar is valid and open.
             self.properties.filetype, innerzips = self.ziprar.get_type()
             if innerzips:
                 self.properties.file_innerzips = True
-                logging.debug("There are inner zips/rars in " + self.absolute_filename)
+                logging.debug("There are inner zips/rars in "
+                              + self.absolute_filename)
 
                 for element in innerzips:
                     # extract the inner zip/rar
-                    logging.debug("Extracting " + element + " inside " + self.absolute_filename)
+                    logging.debug("Extracting " + element
+                                  + " inside " + self.absolute_filename)
                     try:
                         self.ziprar.extract_one_file(element, self.outdir)
-                        logging.debug("Recursively processing " + os.path.join(self.outdir, element))
-                        # TODO: this is probably buggy: we should be providing enable_sql too etc.
-                        droidlysis3.process_file(os.path.join(self.outdir, element), 
-                                                 self.outdir, self.verbose, self.clear, 
-                                                 self.enable_procyon, self.disable_description, self.no_kit_exception)
+                        logging.debug("Recursively processing "
+                                      + os.path.join(self.outdir, element))
+                        # TODO: this is probably buggy: we
+                        # should be providing enable_sql too etc.
+                        droidlysis3.process_file(
+                            os.path.join(self.outdir, element),
+                            self.outdir, self.verbose, self.clear,
+                            self.disable_description,
+                            self.no_kit_exception)
                     except:
-                        logging.warning("Cannot extract %s : %s" % (element, sys.exc_info()[0]))
-            
+                        logging.warning("Cannot extract %s : %s"
+                                        % (element, sys.exc_info()[0]))
+
         if self.properties.filetype == droidutil.APK:
             # our zip actually is an APK
             if not self.clear:
                 # let's unzip
-                logging.debug("Unzipping " + self.absolute_filename + " to " + os.path.join(self.outdir, 'unzipped'))
+                logging.debug("Unzipping " +
+                              self.absolute_filename
+                              + " to " + os.path.join(self.outdir, 'unzipped'))
                 try:
-                    self.ziprar.extract_all(outdir=os.path.join(self.outdir, 'unzipped'))
+                    self.ziprar.extract_all(
+                        outdir=os.path.join(self.outdir, 'unzipped'))
                 except:
-                    logging.warning("Unzipping failed (catching exception): %s" % (sys.exc_info()[0]))
+                    logging.warning("Unzipping failed (catching exception): %s"
+                                    % (sys.exc_info()[0]))
 
         return self.properties.filetype
 
     def disassemble(self):
         """
-        Disassembles the sample (as much as possible), and if the end-user is interested in output (clear unset)
+        Disassembles the sample (as much as possible),
+        and if the end-user is interested in output (clear unset)
         also decompiles it (as much as possible).
-        
+
         APK: 
                 java -jar apktool.jar [-q] d file.apk outdir/apktool
                 if apktool failed
@@ -174,56 +211,62 @@ class droidsample:
                       make sure classes.dex is readable
                       dex2jar classes.dex -o classes-dex2jar.jar
                       unzip -qq classes-dex2jar.jar -d outdir/unjarred
-                      procyon classes-dex2jar.jar -o outdir/procyon
 
         What we'll find:
-        APK, clear unset: ./smali AndroidManifest.xml ./unzipped classes.dex classes-dex2jar.jar ./unjarred ./procyon
+        APK, clear unset: ./smali AndroidManifest.xml ./unzipped classes.dex classes-dex2jar.jar ./unjarred 
         APK, clear set  : ./smali AndroidManifest.xml classes.dex
 
         DEX, clear unset: classes.dex ./smali
-        DEX, clear set  : classes.dex ./smali classes-dex2jar.jar ./unjarred ./procyon
+        DEX, clear set  : classes.dex ./smali classes-dex2jar.jar ./unjarred
 
         """
-        if self.verbose:
-            print("------------- Disassembling")
-            
+        logging.debug("------------- Disassembling")
+
         if self.properties.filetype == droidutil.ARM or \
                 self.properties.filetype == droidutil.RAR or \
                 self.properties.filetype == droidutil.CLASS or \
                 self.properties.filetype == droidutil.UNKNOWN:
-            # TODO: we could be running procyon on a class file.
-            logging.debug("Nothing to disassemble for " + self.absolute_filename)
-            return 
+            logging.debug("Nothing to disassemble for "
+                          + self.absolute_filename)
+            return
 
         if self.properties.filetype == droidutil.APK:
-            # APKTOOL won't output to an existing dir unless you use the -f switch. But then, -f erases the contents
+            # APKTOOL won't output to an existing
+            # dir unless you use the -f switch. But then, -f erases the contents
             # of the output dir... So we can't do this directly on self.outdir
             apktool_outdir = os.path.join(self.outdir, "apktool")
-            logging.debug("Running apktool on inputfile=" + self.absolute_filename)
+            logging.debug("Running apktool on inputfile="
+                          + self.absolute_filename)
 
             if self.verbose:
-                logging.debug("Apktool command: java -jar %s d -f %s %s" % (droidconfig.APKTOOL_JAR,
-                                                                            self.absolute_filename, apktool_outdir))
-                subprocess.call(["java", "-jar", droidconfig.APKTOOL_JAR,
+                logging.debug("Apktool command: java -jar %s d -f %s %s"
+                              % (self.config.APKTOOL_JAR,
+                                 self.absolute_filename, apktool_outdir))
+                subprocess.call(["java", "-jar", self.config.APKTOOL_JAR,
                                  "d", "-f", self.absolute_filename,
                                  "-o", apktool_outdir])
             else:
                 # with quiet option
-                subprocess.call(["java", "-jar", droidconfig.APKTOOL_JAR,
+                subprocess.call(["java", "-jar", self.config.APKTOOL_JAR,
                                  "-q", "d", "-f", self.absolute_filename,
-                                 "-o", apktool_outdir], stdout=self.process_output, stderr=self.process_output)
+                                 "-o", apktool_outdir],
+                                 stdout=self.process_output,
+                                 stderr=self.process_output)
 
             if os.path.isdir(apktool_outdir):
                 droidutil.move_dir(apktool_outdir, self.outdir)
 
             # we want all smali_classes? dir in smali
             if os.path.exists(os.path.join(self.outdir, "smali_classes2")):
-                # we have multidex - we are going to move all smali classes in the same directory
+                # we have multidex - we are going to move
+                # all smali classes in the same directory
                 logging.debug("Moving multidex smali classes to ./smali")
 
-                os.system("cp -R " + os.path.join(self.outdir, "./smali_classes?/*")
+                os.system("cp -R "
+                          + os.path.join(self.outdir, "./smali_classes*/*")
                           + " " + os.path.join(self.outdir, "./smali"))
-                os.system("rm -r " + os.path.join(self.outdir, "./smali_classes?"))
+                os.system("rm -r "
+                          + os.path.join(self.outdir, "./smali_classes*"))
                 
             if self.verbose:
                 logging.debug("Apktool finished")
@@ -232,7 +275,7 @@ class droidsample:
             logging.debug("Extracting classes*.dex")
             try:
                 self.ziprar.extract_one_file('classes.dex', self.outdir)
-                for i in range(2, 10):
+                for i in range(2, 1000):
                     self.ziprar.extract_one_file('classes{}.dex'.format(i), self.outdir)
                     if not os.path.exists(os.path.join(self.outdir, 'classes{}.dex'.format(i))):
                         break
@@ -262,7 +305,7 @@ class droidsample:
                 if os.access(d, os.R_OK):
                     logging.debug("Baksmali on {} -> {}".format(d, smali_dir))
                     try:
-                        subprocess.call(["java", "-jar", droidconfig.BAKSMALI_JAR,
+                        subprocess.call(["java", "-jar", self.config.BAKSMALI_JAR,
                                          "d", "-o", smali_dir, d],
                                         stdout=self.process_output, stderr=self.process_output)
                     except:
@@ -276,48 +319,50 @@ class droidsample:
                 if os.access(d, os.R_OK):
                     jar_file = os.path.join(self.outdir, '{}-dex2jar.jar'.format(
                         os.path.splitext(os.path.basename(d))[0]))
-                    if os.access(droidconfig.DEX2JAR_CMD, os.X_OK):
+                    if os.access(self.config.DEX2JAR_CMD, os.X_OK):
                         if self.verbose:
                             logging.debug("Dex2jar on " + d)
                             try:
-                                subprocess.call([droidconfig.DEX2JAR_CMD, "--force", d, "-o", jar_file],
+                                subprocess.call([self.config.DEX2JAR_CMD, "--force", d, "-o", jar_file],
                                                 stdout=self.process_output, stderr=self.process_output)
                             except:
                                 logging.warning("[-] Dex2jar failed on {}".format(d))
-                        else:
-                            logging.warning("Dex2jar software is not executable, skipping (file: {0})".format(droidconfig.DEX2JAR_CMD))
+                    else:
+                        logging.warning("Dex2jar software is not executable, skipping (file: {0})".format(self.config.DEX2JAR_CMD))
                     
                     if os.access(jar_file, os.R_OK):
-                        if self.enable_procyon and os.access(droidconfig.PROCYON_JAR, os.R_OK):
-                            logging.debug("Procyon decompiler on " + jar_file)
-                            subprocess.call(["java", "-jar", droidconfig.PROCYON_JAR,
-                                            jar_file, "-o", os.path.join(self.outdir, 'procyon')],
-                                            stdout=self.process_output, stderr=self.process_output)
-
-                    logging.debug("Unjarring " + jar_file)
-                    jarziprar = droidziprar.droidziprar(jar_file, zipmode=True, verbose=self.verbose)
-                    if jarziprar.handle is None:
-                        logging.debug("Bad Jar / Failed to unjar " + jar_file)
-                    else:
-                        try:
-                            jarziprar.extract_all(os.path.join(self.outdir, 'unjarred'))
-                        except:
-                            logging.warning("Failed to unjar: %s" % (sys.exc_info()[0]))
+                        logging.debug("Unjarring " + jar_file)
+                        jarziprar = droidziprar.droidziprar(jar_file, zipmode=True, verbose=self.verbose)
+                        if jarziprar.handle is None:
+                            logging.debug("Bad Jar / Failed to unjar " + jar_file)
+                        else:
+                            try:
+                                jarziprar.extract_all(os.path.join(self.outdir, 'unjarred'))
+                            except:
+                                logging.warning("Failed to unjar: %s" % (sys.exc_info()[0]))
                         jarziprar.close()
 
         # Convert binary Manifest
         if self.properties.filetype == droidutil.APK:
             manifest = os.path.join(self.outdir, 'AndroidManifest.xml')
             if not os.access(manifest, os.R_OK) or os.path.getsize(manifest) == 0:
-                logging.warning("Failed to extract binary manifest")
-                """
-                we could attempt to do it with androaxml.py, but that introduces a dependancy to androguard 
-                just for that we could do it with ~/Android/Sdk/tools/bin/apkanalyzer manifest print package.apk, 
-                but that means finding apkanalyzer on the user's host + being sure they have Java 8 as the tool 
-                does not work with Java 11.
-                it's simply not worth doing it, as this case happens very very seldom.
-                """
- 
+                try:
+                    logging.debug( "Extracting binary AndroidManifest.xml")
+                    self.ziprar.extract_one_file('AndroidManifest.xml', self.outdir)
+                    if os.access( manifest, os.R_OK) and os.path.getsize(manifest) > 0:
+                        textmanifest = os.path.join( self.outdir, 'AndroidManifest.text.xml')
+                        subprocess.call( [ "androaxml.py", "--input", manifest, \
+                                           "--output", textmanifest ], \
+                                         stdout=self.process_output, stderr=self.process_output)
+                        if os.access( textmanifest, os.R_OK ):
+                            # overwrite the binary manifest with the converted text one
+                            os.rename(textmanifest, manifest)
+                except AssertionError:
+                    logging.warning("Failed to extract binary manifest: %s" % (sys.exc_info()[0]))
+                
+                except FileNotFoundError:
+                    logging.warning("Error while unzipping or Androguard not installed (no androaxml.py)")
+                     
     def extract_file_properties(self):
         """Extracts file size, 
         nb of dirs and classes in smali dir"""
@@ -359,7 +404,7 @@ class droidsample:
 
             if list:
                 try: 
-                    keyout = subprocess.check_output([droidconfig.KEYTOOL, "-printcert",  "-file",
+                    keyout = subprocess.check_output([self.config.KEYTOOL, "-printcert",  "-file",
                                                       os.path.join(self.outdir, list[0])],
                                                       stderr=self.process_output).decode('utf-8')
                     keyout = re.sub(': ', '#', keyout)
@@ -458,7 +503,7 @@ class droidsample:
                                                                                       filename))
                 missing = True
 
-        if missing and (self.properties.smali['dex_class_loader'] or self.properties.smali['dex_file']):
+        if missing and (self.properties.smali['dex_class_loader'] or self.properties.smali['dex_file'] or self.properties.smali['load_library'] or self.properties.smali['class_loader']):
             self.properties.smali['packed'] = True
 
     def extract_manifest_properties(self):
@@ -523,7 +568,15 @@ class droidsample:
             self.properties.manifest['maxSDK'] = droidutil.get_element(xmldoc, 'uses-sdk', 'android:maxSdkVersion')
             self.properties.manifest['minSDK'] = droidutil.get_element(xmldoc, 'uses-sdk', 'android:minSdkVersion')
             self.properties.manifest['targetSDK'] = droidutil.get_element(xmldoc, 'uses-sdk', 'android:targetSdkVersion')
-            droidutil.get_elements(xmldoc, 'meta-data', 'android:value')
+            tab = droidutil.get_elements(xmldoc, 'meta-data', 'android:name')
+            if self.verbose:
+                logging.debug('meta-data: ' + ''.join(tab))
+            for t in tab:
+                if 'android.content.ContactDirectory' in t:
+                    self.properties.manifest['auto_load'] = True
+                    if self.verbose:
+                        logging.debug('Suspicious ContactDirectory meta-data trick')
+                    break
 
             if self.verbose:
                 logging.debug("MinSDK=%s MaxSDK=%s TargetSDK=%s" % (self.properties.manifest['minSDK'],
@@ -585,6 +638,7 @@ class droidsample:
                                 break # break one level
                         if self.properties.kits[section] is True:
                             break  # break another level
+
         return list
 
     def extract_dex_properties(self):
@@ -710,7 +764,8 @@ class droidsample:
                     if match[mykey]:
                         analysis_file.write("## %s\n" % mykey)
                     for element in match[mykey]:
-                        analysis_file.write("- "+str(element)+"\n")
+                        append = self.extract_method_name(str(element))
+                        analysis_file.write("- "+append+str(element)+"\n")
                     analysis_file.write('\n')
                     analysis_file.close()
 
@@ -721,7 +776,21 @@ class droidsample:
             # all smali properties should then be set to unknown
             for key in sorted(self.properties.smali.keys()):
                 self.properties.smali[key] = 'unknown'
-                                 
+
+    def extract_method_name(self, element):
+        m = re.findall(r'file=(.*?)\s+no=\s*(\d+)', element)
+        if m:
+            file_path, line_no = m[0]
+            with open(file_path, 'r') as f:
+                lines = f.readlines()
+                for i in range(int(line_no), 0, -1):
+                    line = lines[i]
+                    if line.startswith('.method'):
+                        clazz = (lines[0].split(' ')[-1]).strip()
+                        method = (line.split(' ')[-1]).strip()
+                        return f'path={clazz}->{method} '
+        return ''
+
     def extract_wide_properties(self, list_of_kits):
         """Will look for given properties (e.g GPS usage, presence of executables
         in all subdirectories (smali, assets, resources, library...)"""
@@ -735,8 +804,22 @@ class droidsample:
 
         # detecting Flutter debug mode
         flutter_debug = os.path.join(self.outdir, 'unzipped/assets/flutter_assets/kernel_blob.bin')
-        if flutter_debug:
+        if os.access(flutter_debug, os.R_OK):
             self.properties.kits['flutter'] = True
+        
+        # detecting react native framework using index.android.bundle
+        react_assets = os.path.join(self.outdir, 'unzipped/assets/index.android.bundle')
+        if os.access(react_assets, os.R_OK):
+            self.properties.wide['react_asset'] = True
+        
+        # Run "file" to detect hermes bytecode version
+        proc = subprocess.Popen(['file', react_assets], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        output = proc.communicate()[0]
+
+        analysis_file = open(os.path.join(self.outdir, droidlysis3.property_dump_file), 'a')
+        analysis_file.write('# Hermes Bytecode version\n')
+        analysis_file.write(output.decode() + '\n')
+        analysis_file.close()
 
         # detect executables in resources
         self.find_exec_in_resources()
@@ -910,7 +993,7 @@ class droidsample:
                     if found_apk:
                         for apk in found_apk:
                             logging.debug("Recursively processing " + apk)
-                            droidlysis3.process_file(apk, self.outdir, self.verbose, self.clear, self.enable_procyon, 
+                            droidlysis3.process_file(apk, self.outdir, self.verbose, self.clear, 
                                                      disable_report=self.disable_description,
                                                      no_kit_exception=self.no_kit_exception)
 
